/*
 * Copyright 2014-2015 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.
 *
 * This software and the information contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and conditions
 * of a form of NVIDIA software license agreement.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.   This source code is a "commercial item" as
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer  software"  and "commercial computer software
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 *
 * Any use of this source code in individual and commercial software must
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

#ifndef _WINSYS_H_
#define _WINSYS_H_

#include <stdio.h>
#include <stdlib.h>

#ifndef GL_GLEXT_PROTOTYPES
#define GL_GLEXT_PROTOTYPES
#endif
#include <GL/gl.h>
#include <GL/glext.h>
#include <GL/glx.h>

extern void Print(const char *fmt, ...);

struct Info
{
    int width;
    int height;
    GLuint graphicsProgram;
    GLuint graphicsTex;
    GLint graphicsSampler;
    GLint graphicsOffset;
    GLuint computeProgram;
    struct WinInfo * winInfo;
};

#define CHECK(expr, returncode, msg, ...)                                   \
    do {                                                                    \
        if (expr) {                                                         \
            Print("\nERROR: (%s:%d) %s",                                    \
                __FILE__,                                                   \
                __LINE__,                                                   \
                #expr);                                                     \
            Print(msg, __VA_ARGS__);                                        \
            return returncode;                                              \
        }                                                                   \
    } while(0)

#define CHECKV(expr, msg, ...)                                              \
    do {                                                                    \
        if (expr) {                                                         \
            Print("\nERROR: (%s:%d) %s",                                    \
                __FILE__,                                                   \
                __LINE__,                                                   \
                #expr);                                                     \
            Print(msg, __VA_ARGS__);                                        \
            return;                                                         \
        }                                                                   \
    } while(0)

#define CHECKCONTINUE(expr, msg, ...)                                       \
    if (expr) {                                                             \
        Print("\nERROR: (%s:%d) %s",                                        \
            __FILE__,                                                       \
            __LINE__,                                                       \
            #expr);                                                         \
        Print(msg, __VA_ARGS__);                                            \
        continue;                                                           \
    }                                                                       \

#define CHECKGLERRORS()                                                     \
        CheckGlErrors(__FILE__,__LINE__)

#endif //_WINSYS_H_
