/*
 * Copyright 2014-2015 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.
 *
 * This software and the information contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and conditions
 * of a form of NVIDIA software license agreement.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.   This source code is a "commercial item" as
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer  software"  and "commercial computer software
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 *
 * Any use of this source code in individual and commercial software must
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

#ifndef GL_GLEXT_PROTOTYPES
#define GL_GLEXT_PROTOTYPES
#endif

#include "winsys.h"

// =============================================================================
// DEFINES
// =============================================================================

extern const char* NVPM_GL_TYPE;
const char* NVPM_GL_TYPE = "gl";

extern const int NVPM_GL_PRECISION_SHADER_VERSION = 30;
extern const int NVPM_GL_INSTANCE_SHADER_VERSION = 31;
extern const int NVPM_GL_COMPUTE_SHADER_VERSION = 43;

char computeShaderSource[1024] = "#version 430\n";
char vertexShaderInstanceSource[1024] = "#version 140\n";
int g_glversion;

// =============================================================================
// TYPES
// =============================================================================

struct WinInfo
{
    Display* winDisplayX;
    Window winWindowX;
    GLXContext ctx;
};

// ===========================================================================
// CheckGlErrors() - check for OpenGL errors
// ===========================================================================

int CheckGlErrors(const char *file, int line)
{
    int err = 0;
    GLenum glErr;
    while ((glErr = glGetError()) != GL_NO_ERROR)
    {
        Print("\nglError 0x%04x\n at %s:%d\n\n", (int)glErr, file, line);
        err = 1;
    }
    return err;
}

// ===========================================================================
// WinSysInitialize() - create a window
// ===========================================================================

int WinSysInitialize(Info& info, int fullscreen)
{
    info.width = 480;
    info.height = 480;
    info.winInfo = (WinInfo*)malloc(sizeof(WinInfo));

    GLint att[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
    XVisualInfo *vi;
    Colormap cmap;
    Display* xDisplay = XOpenDisplay(NULL);
    CHECK(!xDisplay, 1, "Error: %s failed", "XOpenDisplay");

    Window rootWindow = DefaultRootWindow(xDisplay);
    XSetWindowAttributes xswa;
    xswa.event_mask = ExposureMask | KeyPressMask;

    vi = glXChooseVisual(xDisplay, 0, att);
    cmap = XCreateColormap(xDisplay, rootWindow, vi->visual, AllocNone);
    xswa.colormap = cmap;
    Window xWindow = XCreateWindow(xDisplay, rootWindow, 0, 0, info.width, info.height, 0, vi->depth, InputOutput, vi->visual, CWColormap | CWEventMask, &xswa);

    XMapWindow(xDisplay, xWindow);
    XStoreName(xDisplay, xWindow, "sample");

    info.winInfo->ctx = glXCreateContext(xDisplay, vi, NULL, GL_TRUE);
    glXMakeCurrent(xDisplay, xWindow, info.winInfo->ctx);

    info.winInfo->winDisplayX = xDisplay;
    info.winInfo->winWindowX = xWindow;

    return 0;
}

// ===========================================================================
// WinSysTerminate() - close the window system connection
// ===========================================================================

void WinSysTerminate(Info& info)
{
    if (info.winInfo->winDisplayX)
    {
        XDestroyWindow(info.winInfo->winDisplayX, info.winInfo->winWindowX);
    }
    info.winInfo->winWindowX = 0;

    if (info.winInfo->winDisplayX)
    {
        XCloseDisplay(info.winInfo->winDisplayX);
    }
    info.winInfo->winDisplayX = 0;
    free(info.winInfo);
}

// =============================================================================
// GetCurrentContext() - query the current opengl context
// =============================================================================

uint64_t GetCurrentContext()
{
    return (uint64_t)glXGetCurrentContext();
}

// =============================================================================
// GetGLProcAddress() - get proc address  by name
// =============================================================================

void* GetGLProcAddress (const char* procname)
{
    return (void*) glXGetProcAddress((GLubyte*)procname);
}

// =============================================================================
// SwapBuffers() - Posts colour buffer to the associated native window
// =============================================================================

void SwapBuffers(Info& info)
{
    glXSwapBuffers(info.winInfo->winDisplayX, info.winInfo->winWindowX);
}

// =============================================================================
// SwapBuffers() - nothing to do in opengl context
// =============================================================================

int InitGLState (Info& info)
{
    return 0;
}
